#!/usr/bin/perl5
#
# $Id: cat5.pl,v 1.3 1998/05/21 20:42:22 sheneman Exp $
#
##############################################################################
#
# The contents of this file are subject to the Netscape Public License Version
# 1.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.mozilla.org/NPL/ 
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
# the specific language governing rights and limitations under the License. 
#
# The Initial Developer of the Original Code is Netscape Communications
# Corporation. Portions created by Netscape are Copyright (C) 1998 Netscape
# Communications Corporation. All Rights Reserved.
#
##############################################################################
#
# cat5.pl
#
# This script generates a list of people who are terminated in LDAP and have
#   PeopleSoft records, but are not properly expired in PeopleSoft
#
# AUTHOR:  Luke Sheneman
# HISTORY: 
# 
#   12/09/97 - First implementation
#
# (C)opyright 1997, Sheneman Script Factory, Inc
#

# modules that we need
use Ldapp;                      # LDAP module for Perl5, very cool!
use LdapConf;                   # Generic LDAP config foo
use LdapUtils;                  # Some defaults and utilities we use.

$| = 1; 			# flush all buffers immediately

# set some defaults here
$LDAP_SERVER                  = "hoth.mcom.com";
$LDAP_CERT_PATH               = "/etc/adduser_cert.db";

@ATTRIBUTES = ("uid", "manager", "employeetype", "employeenumber", "cn", "nscppersonexpdate", "userpassword", "objectclass");

# the path to the current files
$LEIF_FILE = "/u/sheneman/projects/cleanup/psoftsync_verbose.txt";
$PATH = "/u/sheneman/projects/cleanup/current";

# the names of the current files
$nscp_file      = "$PATH/nscp_cat5.txt";
$ic_file        = "$PATH/ic_cat5.txt";
$vendors_file   = "$PATH/vendors_cat5.txt";
$interim_file   = "$PATH/interim_cat5.txt";
$oem_file       = "$PATH/oem_cat5.txt";
$unknown_file   = "$PATH/unknown_cat5.txt";


# setup some LDAP parameters for every bind this session
sub setup_ldap {
    # set some defaults for the LDAP bind
    $ld{host} = $LDAP_SERVER;

    # set the cert path
    if($LDAP_CERT_PATH eq "") {
	$ld{port} = $LdapConf::PORT;
    } else {
	$ld{port} = $LdapConf::SSL_PORT;
    }
    $ld{root} = $LdapConf::ROOT;
    $ld{cert} = "$LDAP_CERT_PATH";

    # return success
    return 0;
}


# do a linear search through the exceptions
sub check_in_peoplesoft {

    @results = `grep "No entry in PeopleSoft" $LEIF_FILE | cut -f4 -d" "`;

    $found = 1;   # start by assuming they are in PeopleSoft
    foreach $i (@results) {
	chop $i;  chop $i;

	# print("Comparing [$i] and [$uid]\n");
	if($i eq $uid) {
	    $found = 0;  # if we find them in Leif's output, they are not in PeopleSoft
	    last;
	}
    }

    return $found;
}


# determine if this LDAP account is a "pseudo-account"
sub is_pseudo { 

    $ispseudo = 0;
    foreach $obj (@objectclass) {
	if( ($obj eq "pseudoAccount") || ($obj eq "pseudoaccount") ) {
	    $ispseudo = 1;
	    last;
	}
    }

    return $ispseudo;
}


sub is_person { 

    $isperson = 0;
    foreach $obj (@objectclass) {
	if( ($obj eq "nscpPerson") || ($obj eq "nscpperson") ) {
	    $isperson = 1;
	    last;
	}
    }

    return $isperson;
}



sub is_hidden {
    
    $ishidden = 0;
    foreach $obj (@objectclass) {
	if( ($obj eq "nscpHideThis") || ($obj eq "nscphidethis") ) {
	    $ishidden = 1;
	    last;
	}
    }

    return $ishidden;
}



sub cat5_list {

    # establish the LDAP connection here
    $ld{conn} = new Ldapp($ld{host}, $ld{port}, $ld{bind}, $ld{pswd}, $ld{cert});
    if(!$ld{conn}) { 
	print("Error:  Could not connect to LDAP server $LDAP_SERVER\n");
	return -1;
    }

    open(NSCP_OUT,    ">$nscp_file");
    open(IC_OUT,      ">$ic_file");
    open(VENDORS_OUT, ">$vendors_file");
    open(INTERIM_OUT, ">$interim_file");
    open(OEM_OUT,     ">$oem_file");
    open(UNKNOWN_OUT, ">$unknown_file");

    # prime the files here
    print(NSCP_OUT "\n");
    print(IC_OUT "\n");
    print(VENDORS_OUT "\n");
    print(INTERIM_OUT "\n");
    print(OEM_OUT "\n");
    print(UNKNOWN_OUT "\n");

    #print("Generating list of all users in LDAP.  Please wait...\n");

    $entry = $ld{conn}->search($LdapConf::PEOPLE_ROOT, "onelevel", "uid=*", 0, @ATTRIBUTES);
    while($entry)
    {
	push(@entries, $entry);
	$entry = $ld{conn}->entry;
    }

    #print("Done.\n\n");

    # generate a list of all people with the correct attributes
    foreach $entry (@entries) {
	if($entry)
	{
	    $uid           = @{$entry->{'uid'}}[0];
	    $manager       = @{$entry->{'manager'}}[0];
	    $employeetype  = @{$entry->{'employeetype'}}[0];
	    $employeenumber= @{$entry->{'employeenumber'}}[0];
	    $cn            = @{$entry->{'cn'}}[0];
	    $exp           = @{$entry->{'nscppersonexpdate'}}[0];
	    $mailhost      = @{$entry->{'mailhost'}}[0];
	    $userpassword  = @{$entry->{'userpassword'}}[0];
	    @objectclass   = @{$entry->{'objectclass'}};

	    # see if we are hidden
	    $ishidden = &is_hidden();
	    
	    # see if we are a person or not
	    $ispseudo = &is_pseudo();


	    if( ($userpassword =~ "TERMINATED") && !$ispseudo && !$ishidden) {
		$result = &check_in_peoplesoft;
		if($result && !$exp) {

		    # determine how the next lines will be outputed
		    if   ($employeetype eq "Employee")   { $OUT = NSCP_OUT; }
		    elsif($employeetype eq "Vendor")     { $OUT = VENDORS_OUT; }
		    elsif($employeetype eq "Interim")    { $OUT = INTERIM_OUT; }
		    elsif($employeetype eq "OEM Partner"){ $OUT = OEM_OUT; }
		    elsif($employeetype eq "Contractor") { $OUT = IC_OUT; }
		    else { $OUT = UNKNOWN_OUT; }

		    print($OUT "USER:         $uid\n");
		    print($OUT "  FULL NAME:  $cn\n");
		    print($OUT "  EMP TYPE:   $employeetype\n");
		    print($OUT "  EMP NUMBER: $employeenumber\n");
		    print($OUT "  MANAGER:    $manager\n");
		    print($OUT "  MAILHOST:   $mailhost\n");
		    print($OUT "  PASSWORD:   $userpassword\n");
		    print($OUT "  EXPIRATION: $exp\n");
		    print($OUT "\n\n");
		}
	    }
	}
    }

    # close the LDAP connection here
    if($ld{conn}) {
	$ld{conn}->close;
    }

    close(NSCP_OUT);
    close(IC_OUT);
    close(VENDORS_OUT);
    close(INTERIM_OUT);
    close(OEM_OUT);
    close(UNKNOWN_OUT);
	
    return 0;
}


# setup LDAP connection information
&setup_ldap;

&cat5_list;


exit 0;




